@@ -1,5 +1,5 @@
 /* krb5/cred.c --- Kerberos 5 GSS-API credential management functions.
- * Copyright (C) 2003, 2004, 2005, 2006, 2007  Simon Josefsson
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009  Simon Josefsson
  *
  * This file is part of the Generic Security Service (GSS).
  *
@@ -64,7 +64,13 @@ acquire_cred1 (OM_uint32 * minor_status,
   {
     char *p;
 
-    p = xmalloc (k5->peerptr->length + 1);
+    p = malloc (k5->peerptr->length + 1);
+    if (!p)
+      {
+	if (minor_status)
+	  *minor_status = ENOMEM;
+	return GSS_S_FAILURE;
+      }
     memcpy (p, k5->peerptr->value, k5->peerptr->length);
     p[k5->peerptr->length] = 0;
 
@@ -98,19 +104,31 @@ gss_krb5_acquire_cred (OM_uint32 * minor_status,
   OM_uint32 maj_stat;
   gss_cred_id_t p = *output_cred_handle;
 
+  p->krb5 = calloc (sizeof (*p->krb5), 1);
+  if (!p->krb5)
+    {
+      if (minor_status)
+	*minor_status = ENOMEM;
+      return GSS_S_FAILURE;
+    }
+
   if (actual_mechs)
     {
       maj_stat = gss_create_empty_oid_set (minor_status, actual_mechs);
       if (GSS_ERROR (maj_stat))
-	return maj_stat;
+	{
+	  free (p->krb5);
+	  return maj_stat;
+	}
       maj_stat = gss_add_oid_set_member (minor_status, GSS_KRB5,
 					 actual_mechs);
       if (GSS_ERROR (maj_stat))
-	return maj_stat;
+	{
+	  free (p->krb5);
+	  return maj_stat;
+	}
     }
 
-  p->krb5 = xcalloc (sizeof (*p->krb5), 1);
-
   maj_stat = acquire_cred1 (minor_status, desired_name, time_req,
 			    desired_mechs, cred_usage,
 			    &p, actual_mechs, time_rec);
